# Micro Data Analysis 2025
# CLASS 2
# Binary / ordered responses
# Wiktor Budzinski

# ctrl+l
rm(list=ls())
setwd("C:\\Users\\budzi\\Documents\\Teaching\\MDA 2025\\Class 2")        # sets the access path (sometimes change the characters '\' to '/' is needed)

# Installing required packages
packages = c("psych",  "MASS", "oglmx", "pscl")
find.package(packages)

# if the library is not installed on the given computer, delete the first '#'
# install.packages("psych")
# install.packages("MASS")
# install.packages("oglmx")
# install.packages("pscl")
rm(packages)

# Loading libraries
library(psych)
library(statmod)
# library(dummies)
library(MASS)
library(mfx)
library(oglmx)
library(pscl)
library(msm)
library(nlWaldTest)
library(glmx)
library(blorr)
library(Rchoice)

################################################# Exercise 1 #########################################
# Binary data

oil  =  readRDS("me.oil1.rds")
View(oil)
hist(oil$bid)

Eq1 <- ans ~ bid + linc + notax +ccoast + protest
model1 <- glm(Eq1, data = oil, family=binomial(link="logit"))
summary(model1)

# Testing functional form:
blr_linktest(model1)

# Marginal effects for logit
me1 <- logitmfx(Eq1, data = oil, atmean = FALSE)
me1


# WTP from the simple model
model1a <- glm(ans ~ bid, data = oil, family=binomial(link="logit"))
summary(model1a)
nlConfint(model1a, "-b[1]/b[2]")

# WTP for model with additional variables:

nlConfint(model1, "-b[1]/b[2]")
mean(oil$linc); 

nlConfint(model1, "-(b[1]+b[3]*10.43795)/b[2]") # average income
nlConfint(model1, "-(b[1]+b[3]*10.43795 + b[5])/b[2]") #at the coast
nlConfint(model1, "-(b[1]+b[3]*10.43795 + b[6])/b[2]") #Protesters

# Plot: Income vs. WTP
coefs<- model1$coefficients
plot(exp(seq(7.8, 12.3, by = 0.1)), -(coefs[1] + coefs[3]*seq(7.8, 12.3, by = 0.1))/coefs[2])

# Different types of functional forms
model2 <- glm(Eq1, data = oil, family=binomial(link="probit"))
summary(model2)

model3 <- glm(Eq1, data = oil, family=binomial(link="cloglog"))
summary(model3)

extractAIC(model1); extractAIC(model2); extractAIC(model3)

# Heteroskedastic binary models

Eq2 <- ans ~ bid + linc + notax +ccoast + protest | ccoast + bid
model4 <- hetprob(Eq2, data = oil, link = "logit")
summary(model4)

model5 <- hetprob(Eq2, data = oil, link = "probit")
summary(model5)

################################################# Exercise 2 ########################################
# Ordered models

rm(list=ls())
baltic  <-  readRDS("me.baltic.rds")
baltic <- baltic[baltic$ruo == 0,]
baltic$age <- baltic$age/10
hist(baltic$envw)

Eq1 <- factor(envw) ~ dk + ee + fi + lit + lv + pl + ruc + se + bocc + age + male + factor(edu) + hhsize + hinc + ldist
model1 <- polr(Eq1, data=baltic, Hess=TRUE, method = c("logistic"))
summary(model1)

# Alternative method (with threshold parameters)
Eq1b <-  envw ~ dk + ee + fi + lit + lv + pl + ruc  + se + bocc + age + male + factor(edu) + hhsize + hinc + ldist
model1b <- ologit.reg(Eq1b, data=baltic)
summary(model1b)
# Interpretation
# People whose profession is associated with the Baltic Sea are more worried (on average) about the state of its environment

# Marginal effects
options(scipen=999) # disable scientific notation
margins.oglmx(model1b, AME=TRUE) # average marginal effects

margins.oglmx(model1b) # at mean
margins.oglmx(model1b, ascontinuous = TRUE)  # binary variable are treated as if continous

# Similarly, you can estimate an ordered probit

model2 <- oprobit.reg(Eq1b, data=baltic)
summary(model2) 

################################################# Exercise 2 ########################################
# Heteroskedasticity

model3 <-oglmx(Eq1b, ~dk + ee + fi + lit + lv + pl + ruc  + se, data=baltic,  constantSD=FALSE, constantMEAN=FALSE)
summary(model3) 